import { mapObj, pipe } from './fn.js'

export const Reset = Object.freeze({
    All: 0,
    Intensity: 22,
    Italic: 23,
    Underline: 24,
    Blink: 25,
    Reverse: 27,
    Conceal: 28,
    Strike: 29
})

export const Style = Object.freeze({
    Bold: 1,
    Faint: 2,
    Italic: 3,
    Underline: 4,
    SlowBlink: 5,
    RapidBlink: 6,
    ReverseVideo: 7,
    Conceal: 8,
    Strike: 9
})

const ForegroundColor = Object.freeze({
    Black: 30,
    Red: 31,
    Green: 32,
    Yellow: 33,
    Blue: 34,
    Magenta: 35,
    Cyan: 36,
    White: 37,
    Default: 39
})

const BackgroundColor = mapObj(
    ([k, v]) => [k, v + 10]
)(ForegroundColor)

const sgr = args => `\x1b[${args.join(';')}m`

export const Color = Object.freeze({
    Foreground: ForegroundColor,
    Background: BackgroundColor,

    get fg() { return this.Foreground },
    get bg() { return this.Background }
})

export const prepend = args => s => `${sgr(args)}${s}`
export const append = args => s => `${s}${sgr(args)}`
export const wrap = (start, end) => pipe(prepend(start), append(end))

